unit CommunicationServer;

interface

uses
  Windows, Messages, QDialogs, SysUtils, Classes, ScktComp, IdTrivialFTPServer, helptools;

type
  TReadTextEvent = procedure(Sender: TObject; Text: string; ClientName: string) of object;
  TReadFileEvent = procedure(Sender: TObject; Filename: string; ClientName: string) of object;

type
  TCommunicationServer = class(TServerSocket)
  private
    FTPServer :  TIdTrivialFTPServer;
    ConnectionsArray : Array of string;
    FConnection_count : integer;
    FOnClientConnectCount: TNotifyEvent;
    FOnClientReadText: TReadTextEvent;
    FOnClientReadFile: TReadFileEvent;
    procedure ClientConnected(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientDisconnected(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientReadData(Sender: TObject; Socket: TCustomWinSocket);
    function CheckConnection(ConnectionNumber : integer) : bool;
  protected

  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Start();
    procedure SendText(ClientName : string; Text: string);
    procedure SendFile(ClientName : string; localname: string; remotename : string);
    procedure GetFile(ClientName : string; remotename : string; localname : string);
  published
    property Connection_count : integer read FConnection_count;
    property OnClientReadText : TReadTextEvent read FOnClientReadText write FOnClientReadText;
    property OnClientReadFile : TReadFileEvent read FOnClientReadFile write FOnClientReadFile;
    property OnClientConnectCount : TNotifyEvent read FOnClientConnectCount write FOnClientConnectCount;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Gestapo', [TCommunicationServer]);
end;

{ TCommunicationServer }

constructor TCommunicationServer.Create(AOwner: TComponent);
begin
  inherited;
  Port := 10079;
  Active := false;
  SetLength(ConnectionsArray,20);
  FConnection_count := 0;
  ServerType := stNonBlocking;
  OnClientConnect :=  ClientConnected;
  OnClientDisconnect := ClientDisconnected;
  Socket.OnClientRead := ClientReadData;
  FTPServer := TIdTrivialFTPServer.Create(self);
  FTPServer.DefaultPort := 69;
  FTPServer.ThreadedEvent := true;
end;

destructor TCommunicationServer.Destroy;
begin
  FTPServer.Free;
  inherited;
end;

procedure TCommunicationServer.ClientConnected(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  self.Socket.Connections[FConnection_count].SendText('ADD_NAME ' + IntToStr(FConnection_count));
end;

procedure TCommunicationServer.ClientDisconnected(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  Socket.SendText('REMOVE_NAME');
end;

procedure TCommunicationServer.ClientReadData(Sender: TObject;
  Socket: TCustomWinSocket);
var
  SenderAddress : string;
  ReceivedString : string;
  Command : string;
  ClientName : string;
  ConnectionNumber : integer;
//  IDName : string;
  i,ii : integer;
begin
  SenderAddress := Socket.RemoteAddress;
  ReceivedString := Socket.ReceiveText;

  Command := ParseString(ReceivedString);

  if Command = 'TEXT' then
    if Assigned(OnClientReadText) Then
    begin
      ClientName := ParseString(ReceivedString);
      OnClientReadText(self,ReceivedString,ClientName);
    end;
  if Command = 'FILE' then
    if Assigned(OnClientReadFile) Then
    begin
      ClientName := ParseString(ReceivedString);
      OnClientReadFile(self,ReceivedString,ClientName);
    end;
  if Command = 'ADD_NAME' then
  begin
    FConnection_count := FConnection_count + 1;
    ConnectionNumber := StrToInt(ParseString(ReceivedString));
    ConnectionsArray[ConnectionNumber]  := ReceivedString;  // Client Identification Name
    if Assigned(OnClientConnectCount) Then
      OnClientConnectCount(self);
  end;
  if Command = 'REMOVE_NAME' then
  begin
    if Connection_count = 0 then exit;                 // if there is no connections the exit
    for i:= 0 to FConnection_count-1 do                // go from beginning to the searched name
      if ConnectionsArray[i] = ReceivedString then     // is arrays name = searched name
        break;                                         // searched index has been found
    for ii:=i to FConnection_count-1 do                // go to the end of array
      ConnectionsArray[ii] := ConnectionsArray[ii+1];  // remove names down by one
    FConnection_count := FConnection_count - 1;        // one connection has been removed
    if Assigned(OnClientConnectCount) Then
      OnClientConnectCount(self);
  end;
end;

procedure TCommunicationServer.GetFile(ClientName : string; remotename : string; localname: string);
var index : integer;
begin
  if Connection_count = 0 then exit;                     // if there is no connections the exit
  for index:= 0 to FConnection_count-1 do                // go from beginning to the searched name
    if ConnectionsArray[index] = ClientName then         // is arrays name = searched name
      break;                                             // searched index has been found
  if not CheckConnection(index) then exit;
  Socket.Connections[index].SendText('SEND ' + remotename + ' ' + localname);
  if Assigned(OnClientReadFile) then
      OnClientReadFile(self,localname,ClientName);
end;

procedure TCommunicationServer.SendFile(ClientName : string; localname: string; remotename : string);
var index : integer;
begin
  if Connection_count = 0 then exit;                     // if there is no connections the exit
  for index:= 0 to FConnection_count-1 do                // go from beginning to the searched name
    if ConnectionsArray[index] = ClientName then         // is arrays name = searched name
      break;                                             // searched index has been found
  if not CheckConnection(index) then exit;
  Socket.Connections[index].SendText('RECEIVE ' + localname + ' ' + remotename);
end;


procedure TCommunicationServer.SendText(ClientName : string; Text: string);
var index : integer;
begin
  if Connection_count = 0 then exit;                     // if there is no connections the exit
  for index:= 0 to FConnection_count-1 do                // go from beginning to the searched name
    if ConnectionsArray[index] = ClientName then         // is arrays name = searched name
      break;                                             // searched index has been found
  if not CheckConnection(index) then exit;
  Socket.Connections[index].SendText('TEXT ' + Text);
end;

procedure TCommunicationServer.Start;
begin
  Open;
  FTPServer.Active := true;
end;

function TCommunicationServer.CheckConnection(
  ConnectionNumber: integer): bool;
begin
  Result := (0 <= ConnectionNumber) and (ConnectionNumber < Connection_count);
end;

end.
