unit DataController;

interface

uses
  Windows, Messages, SysUtils, Classes, helptools;

type
  TDataController = class(TComponent)
  private
    F1 : TextFile;
    procedure Save(filename : string; line : string);
    procedure CreateFile(filename : string);
    procedure RemoveFile(filename : string);
    procedure CreateDirectory(path : string);
    procedure RemoveDirectory(path : string);
    procedure ChangeDirectory(path : string);
  protected
    { Protected declarations }
  public
    logpath : string;
    configpath : string;
    logextension : string;
    cfgextension : string;
    pictureextension : string;
    logfile : string;
    separator : char;

    constructor Create(AOwner : TComponent); override;
    function  GetDirectoryNames(path : string) : TStringList;
    procedure SaveToLog(line : string; ClientName : string);
    function  ReadFromLog(line : string; ClientName : string) : TStringList;
    procedure SaveToConfig(line : string; ClientName : string);
    function  GiveFileNames(path : string; mask : string) : TStringList;
  published
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Gestapo', [TDataController]);
end;

{ TDataController }

//changes into specified directory
procedure TDataController.ChangeDirectory(path: string);
begin
  if not SetCurrentDir(path) then
    raise Exception.Create('Can''t change into directory ' + path);
end;

constructor TDataController.Create(AOwner: TComponent);
begin
  inherited;
end;

//creates a directory which is specified in path parameter
procedure TDataController.CreateDirectory(path : string);
begin
  if not DirectoryExists(path) then
    if not CreateDir(path) then
      raise Exception.Create('Cannot create ' + path);
end;

//creates a file specifies in filename parameter into current directory
procedure TDataController.CreateFile(filename: string);
var filehandle : integer;
begin
  if FileExists(filename) then
  begin
    raise Exception.Create('File ' + filename + ' already exists');
    exit;
  end;
  filehandle := FileCreate(filename);
  if filehandle = -1 then
    raise Exception.Create('Unable to create file ' + filename);
  fileclose(filehandle);
end;

//return currents directorys directory names in a StringList
function TDataController.GetDirectoryNames(path: string): TStringList;
var f1 : TSearchRec;
    list : TStringList;
begin
  list := TSTringList.Create;
  FindFirst(path + '*.*',faDirectory,f1);
  while true Do Begin
    if FindNext(f1) <> 0 then break;
    if (f1.Name <> '.') and (f1.Name <> '..') and (f1.Attr = faDirectory ) then
      list.Add(f1.Name);
  end;
  FindClose(f1);
  result := List;
end;

function TDataController.GiveFileNames(path : string; mask : string): TStringList;
var list : TStringList;
    oldpath : string;
    frec1,frec2 : TSearchRec;
begin
  oldpath := GetCurrentDir;
  list := TStringList.Create;
  ChangeDirectory(path);

  FindFirst('.\' + '*.*',faDirectory,frec1);
  while true Do Begin
    if FindNext(frec1) <> 0 then break;
    if (frec1.Name <> '.') and (frec1.Name <> '..') and (frec1.Attr = faDirectory ) then begin
      ChangeDirectory('.\' + frec1.Name + '\');
      if FindFirst('.\' + mask,faAnyFile,frec2) = 0 then
        list.Add(GetCurrentDir + '\' + frec2.Name);
      while true Do Begin
        if FindNext(frec2) <> 0 then break;
        list.Add(GetCurrentDir + '\' + frec2.Name);
      end;
      ChangeDirectory('..\');
      FindClose(frec2);
    end;

  end;
  FindClose(frec1);
  SetCurrentDir(oldpath);
  result := list;
end;

function TDatacontroller.ReadFromLog(line: string; ClientName : string) : TStringList;
var sensorname : string;
    data : string;
    help : string;
    list : TStringList;
    oldpath : string;
begin
  oldpath := GetCurrentDir;
  ChangeDirectory(logpath + ClientName + '\');
  sensorname := ParseString(line,separator);
  if not FileExists(sensorname + logextension) then begin
    ChangeDirectory(oldpath);
    result := nil;
    exit;
  end;

  data := line;

  list := TStringList.Create;
  AssignFile(F1,sensorname + logextension);
  Reset(f1);
  if data = 'LAST' then begin
    while not Eof(F1) do
      Readln(f1,help);
    list.Add(help);
  end;

  if data = 'ALL' then begin
    while not Eof(F1) do begin
      Readln(f1,help);
      list.Add(help);
    end;
  end;
  CloseFile(f1);
  SetCurrentDir(oldpath);
  result := list;
end;

//removes directory which is given in the path parameter
procedure TDataController.RemoveDirectory(path: string);
begin
  if not DirectoryExists(path) then
    exit;
  if not RemoveDir(path) then
    raise Exception.Create('Cannot remove ' + path);
end;

//removes file which is given in the filename parameter
procedure TDataController.RemoveFile(filename: string);
begin
  if FileExists(FileName) then
    DeleteFile(filename);
end;

//appends a line to a file
procedure TDataController.Save(filename : string; line: string);
begin
  if not FileExists(FileName) then
  begin
    raise Exception.Create('File does not exist!');
    exit;
  end;
  AssignFile(F1,filename);
  Append(F1);
  Writeln(F1,line);
  Flush(F1);
  CloseFile(F1);
end;

procedure TDataController.SaveToConfig(line: string; ClientName : string);
var oldpath : string;
begin
  oldpath := getcurrentdir;
  if not DirectoryExists(configpath) then
    CreateDirectory(configpath);
  ChangeDirectory(configpath);
  setcurrentdir(oldpath);
end;

//Appends line to logfile. Unitname is directory and sensor is its filename
//data contains the information about the sensor.
procedure TDataController.SaveToLog(line: string; ClientName : string);
var sensor : string;
    data : string;
    oldpath : string;
begin
  oldpath := getcurrentdir;
  sensor := ParseString(line,separator);
  data := line;

  if not DirectoryExists(logpath) then
    CreateDirectory(logpath);
  ChangeDirectory(logpath);

  if not setcurrentdir('.\' + ClientName + '\') then begin
    self.CreateDirectory('.\' + ClientName + '\');
    setcurrentdir('.\' + ClientName + '\');
  end;

  if not FileExists(logfile + logextension) then   //creates log.dat file
    CreateFile(logfile + logextension);
  save(logfile + logextension,data + ' ' + sensor);

  if not FileExists(sensor + logextension) then
    CreateFile(sensor + logextension);
  save(sensor + logextension,data);
  changedirectory(oldpath);
end;

end.
