unit DataController;

interface

uses
  Windows, Messages, SysUtils, Classes, helptools;

const logpath : string = '.\log\';
const configpath : string = '.\cfg\';
const logextension : string = '.dat';
const separator : char = ' ';

type
  TDataController = class(TComponent)
  private
    F1 : TextFile;
    directories : TList;
    procedure Save(filename : string; line : string);
    procedure CreateFile(filename : string);
    procedure RemoveFile(filename : string);
    procedure CreateDirectory(path : string);
    procedure RemoveDirectory(path : string);
    procedure ChangeDirectory(path : string);
    procedure SaveToLog(line : string);
    procedure SaveToConfig(line : string);
    function ReadFromLog(line : string) : string;
  protected
    { Protected declarations }
  public
    constructor Create(AOwner : TComponent); override;
    function ProcessCommand(line : string) : string;
    function GetDirectoryNames(path : string) : TStringList;
  published
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Gestapo', [TDataController]);
end;

{ TDataController }

procedure TDataController.ChangeDirectory(path: string);
begin
  if not SetCurrentDir(path) then
    raise Exception.Create('Can''t change into directory ' + path);
end;

constructor TDataController.Create(AOwner: TComponent);
begin
  inherited;
end;

procedure TDataController.CreateDirectory(path : string);
begin
  if not DirectoryExists(path) then
    if not CreateDir(path) then
      raise Exception.Create('Cannot create ' + path);
end;

procedure TDataController.CreateFile(filename: string);
var filehandle : integer;
begin
  if FileExists(filename) then
  begin
    raise Exception.Create('File already exists');
    exit;
  end;
  filehandle := FileCreate(filename);
  if filehandle = -1 then
    raise Exception.Create('Unable to create file ' + filename);
  fileclose(filehandle);
end;

function TDataController.GetDirectoryNames(path: string): TStringList;
var f1 : TSearchRec;
    list : TStringList;
begin
  list := TSTringList.Create;
  FindFirst(path + '*.*',faDirectory,f1);
  while true Do Begin
    if FindNext(f1) <> 0 then break;
    if (f1.Name <> '.') and (f1.Name <> '..') and (f1.Attr = faDirectory ) then
      list.Add(f1.Name);
  end;
  FindClose(f1);
  result := List;
end;

function TDataController.ProcessCommand(line: string) : string;
var command : string;
begin
  command := ParseString(line,separator);
  if command = 'SAVELOG' then
    SaveToLog(line);
  if command = 'SAVECONFIG' then
    SaveToConfig(line);
  if command = 'READLOG' then
    result := ReadFromLog(line);
end;


function TDatacontroller.ReadFromLog(line: string) : string;
var unitname : string;
    sensorname : string;
    oldpath : string;
begin
  oldpath := GetCurrentDir;
  unitname := ParseString(line,separator);
  ChangeDirectory(logpath + unitname + '\');
  sensorname := ParseString(line,separator);
  AssignFile(F1,sensorname + logextension);
  Readln
  SetCurrentDir(oldpath);
end;

procedure TDataController.RemoveDirectory(path: string);
begin
  if not DirectoryExists(path) then
    exit;
  if not RemoveDir(path) then
    raise Exception.Create('Cannot remove ' + path);
end;

procedure TDataController.RemoveFile(filename: string);
begin
  if FileExists(FileName) then
    DeleteFile(filename);
end;

procedure TDataController.Save(filename : string; line: string);
begin
  if not FileExists(FileName) then
  begin
    raise Exception.Create('File does not exist!');
    exit;
  end;
  AssignFile(F1,filename);
  Append(F1);
  Writeln(F1,line);
  Flush(F1);
  CloseFile(F1);
end;

procedure TDataController.SaveToConfig(line: string);
var unitname : string;
    oldpath : string;
begin
  oldpath := getcurrentdir;
  unitname := ParseString(line,separator);
  if not DirectoryExists(configpath) then
    CreateDirectory(configpath);
  ChangeDirectory(configpath);
  setcurrentdir(oldpath);
end;

procedure TDataController.SaveToLog(line: string);
var unitname : string;
    sensor : string;
    data : string;
    oldpath : string;
begin
  oldpath := getcurrentdir;
  unitname := ParseString(line,separator);
  sensor := ParseString(line,separator);
  data := line;
  if not DirectoryExists(logpath) then
    CreateDirectory(logpath);
  ChangeDirectory(logpath);
  if not setcurrentdir('.\' + unitname + '\') then begin
    self.CreateDirectory('.\' + unitname + '\');
    setcurrentdir('.\' + unitname + '\');
  end;
  if not FileExists(sensor + logextension) then
    CreateFile(sensor + logextension);
  save(sensor + logextension,data);
  changedirectory(oldpath);
end;

end.


